    "br.com",	"whois.centralnic.net",
    "cn.com",	"whois.centralnic.net",
    "de.com",	"whois.centralnic.net",
    "eu.com",	"whois.centralnic.net",
    "gb.com",	"whois.centralnic.net",
    "gb.net",	"whois.centralnic.net",
    "gr.com",	"whois.centralnic.net",
    "hu.com",	"whois.centralnic.net",
    "in.net",	"whois.centralnic.net",
    "no.com",	"whois.centralnic.net",
    "qc.com",	"whois.centralnic.net",
    "ru.com",	"whois.centralnic.net",
    "sa.com",	"whois.centralnic.net",
    "se.com",	"whois.centralnic.net",
    "se.net",	"whois.centralnic.net",
    "uk.com",	"whois.centralnic.net",
    "uk.net",	"whois.centralnic.net",
    "us.com",	"whois.centralnic.net",
    "uy.com",	"whois.centralnic.net",
    "za.com",	"whois.centralnic.net",
    "jpn.com",	"whois.centralnic.net",
    "web.com",	"whois.centralnic.net",
    "com",	"\x04" "whois.verisign-grs.com",
    "za.net",	"whois.za.net",
    "net",	"\x04" "whois.verisign-grs.com",
    "eu.org",	"whois.eu.org",
    "za.org",	"whois.za.org",
    "org",	"whois.pir.org",
    "edu",	"whois.educause.edu",
    "gov",	"whois.dotgov.gov",
    "int",	"whois.iana.org",
    "mil",	"\x03",
    "e164.arpa",	"whois.ripe.net",
    "in-addr.arpa",	"\x0C",
    "arpa",	"whois.iana.org",
    "aero",	"whois.aero",
    "asia",	"whois.nic.asia",
    "biz",	"whois.biz",
    "cat",	"whois.nic.cat",
    "coop",	"whois.nic.coop",
    "info",	"whois.afilias.net",
    "jobs",	"\x04" "whois.nic.jobs",
    "mobi",	"whois.afilias.net",
    "museum",	"whois.museum",
    "name",	"whois.nic.name",
    "post",	"whois.dotpostregistry.net",
    "pro",	"whois.afilias.net",
    "tel",	"whois.nic.tel",
    "travel",	"whois.nic.travel",
    "xxx",	"whois.nic.xxx",
    "ac",	"whois.nic.ac",
    "ad",	"\x03",
    "ae",	"whois.aeda.net.ae",
    "af",	"whois.nic.af",
    "ag",	"whois.nic.ag",
    "ai",	"whois.ai",
    "al",	"\x03",
    "am",	"whois.amnic.net",
    "ao",	"\x03",
    "aq",	"\x03",
    "ar",	"\x01http://www.nic.ar/",
    "as",	"whois.nic.as",
    "priv.at",	"whois.nic.priv.at",
    "at",	"whois.nic.at",
    "au",	"whois.audns.net.au",
    "aw",	"whois.nic.aw",
    "ax",	"whois.ax",
    "az",	"\x01http://www.nic.az/",
    "ba",	"\x01http://www.nic.ba/stream/whois/",
    "bb",	"\x01http://whois.telecoms.gov.bb/search_domain.php",
    "bd",	"\x01http://www.registry.com.bd/whois.php",
    "be",	"whois.dns.be",
    "bf",	"\x03",
    "bg",	"whois.register.bg",
    "bh",	"\x03",
    "bi",	"whois1.nic.bi",
    "bj",	"whois.nic.bj",
    "bm",	"\x01http://www.bermudanic.bm/cgi-bin/lansaweb?procfun+BMWHO+BMWHO2+WHO",
    "bn",	"whois.bnnic.bn",
    "bo",	"whois.nic.bo",
    "br",	"whois.registro.br",
    "bs",	"\x01http://www.nic.bs/cgi-bin/search.pl",
    "bt",	"\x01http://www.nic.bt/",
    "bv",	"\x03",
    "by",	"whois.cctld.by",
    "bw",	"whois.nic.net.bw",
    "bz",	"\x08",
    "co.ca",	"whois.co.ca",
    "ca",	"whois.cira.ca",
    "cc",	"\x04" "ccwhois.verisign-grs.com",
    "cd",	"whois.nic.cd",
    "cf",	"whois.dot.cf",
    "cg",	"\x03",
    "ch",	"whois.nic.ch",
    "ci",	"whois.nic.ci",
    "ck",	"\x03",
    "cl",	"whois.nic.cl",
    "cm",	"whois.netcom.cm",
    "edu.cn",	"whois.edu.cn",
    "cn",	"whois.cnnic.cn",
    "uk.co",	"whois.uk.co",
    "co",	"whois.nic.co",
    "cr",	"whois.nic.cr",
    "cu",	"\x01http://www.nic.cu/",
    "cv",	"\x01http://www.dns.cv/",
    "cw",	"\x03",
    "cx",	"whois.nic.cx",
    "cy",	"\x01http://www.nic.cy/nslookup/online_database.php",
    "cz",	"whois.nic.cz",
    "de",	"whois.denic.de",
    "dj",	"\x01http://www.nic.dj/whois.php",
    "dk",	"whois.dk-hostmaster.dk",
    "dm",	"whois.nic.dm",
    "do",	"\x01http://www.nic.do/whois-h.php3",
    "dz",	"whois.nic.dz",
    "ec",	"whois.nic.ec",
    "ee",	"whois.tld.ee",
    "eg",	"\x03",
    "er",	"\x03",
    "es",	"\x01https://www.nic.es/",
    "et",	"\x03",
    "eu",	"whois.eu",
    "fi",	"whois.fi",
    "fj",	"whois.usp.ac.fj",
    "fk",	"\x03",
    "fm",	"\x01http://dot.fm/whois.html",
    "fo",	"whois.nic.fo",
    "fr",	"whois.nic.fr",
    "ga",	"whois.dot.ga",
    "gb",	"\x03",
    "gd",	"whois.nic.gd",
    "ge",	"\x01http://www.registration.ge/",
    "gf",	"\x01https://www.dom-enic.com/whois.html",
    "gg",	"whois.gg",
    "gh",	"\x01http://www.nic.gh/customer/search_c.htm",
    "gi",	"whois2.afilias-grs.net",
    "gl",	"whois.nic.gl",
    "gm",	"\x01http://www.nic.gm/htmlpages/whois.htm",
    "gn",	"\x03",
    "gp",	"\x01https://www.dom-enic.com/whois.html",
    "gq",	"whois.dominio.gq",
    "gr",	"\x01https://grweb.ics.forth.gr/public/whois.jsp?lang=en",
    "gs",	"whois.nic.gs",
    "gt",	"\x01http://www.gt/",
    "gu",	"\x01http://gadao.gov.gu/domainsearch.htm",
    "gw",	"\x01http://nic.gw/en/whois/",
    "gy",	"whois.registry.gy",
    "hk",	"whois.hkirc.hk",
    "hm",	"whois.registry.hm",
    "hn",	"whois.nic.hn",
    "hr",	"whois.dns.hr",
    "ht",	"whois.nic.ht",
    "hu",	"whois.nic.hu",
    "id",	"whois.pandi.or.id",
    "ie",	"whois.domainregistry.ie",
    "il",	"whois.isoc.org.il",
    "im",	"whois.nic.im",
    "in",	"whois.inregistry.net",
    "io",	"whois.nic.io",
    "iq",	"whois.cmc.iq",
    "ir",	"whois.nic.ir",
    "is",	"whois.isnic.is",
    "it",	"whois.nic.it",
    "je",	"whois.je",
    "jm",	"\x03",
    "jo",	"\x01http://www.dns.jo/Whois.aspx",
    "jp",	"whois.jprs.jp",
    "ke",	"whois.kenic.or.ke",
    "kg",	"whois.domain.kg",
    "kh",	"\x03",
    "ki",	"whois.nic.ki",
    "km",	"\x03",
    "kn",	"whois.nic.kn",
    "kp",	"\x03",
    "kr",	"whois.kr",
    "kw",	"\x01http://www.kw/",
    "ky",	"whois.kyregistry.ky",
    "kz",	"whois.nic.kz",
    "la",	"whois.nic.la",
    "lb",	"\x01http://www.aub.edu.lb/lbdr/",
    "lc",	"whois2.afilias-grs.net",
    "li",	"whois.nic.li",
    "lk",	"whois.nic.lk",
    "lr",	"\x03",
    "ls",	"\x01http://www.co.ls/co.asp",
    "lt",	"whois.domreg.lt",
    "lu",	"whois.dns.lu",
    "lv",	"whois.nic.lv",
    "ly",	"whois.nic.ly",
    "ma",	"whois.registre.ma",
    "mc",	"\x03",
    "md",	"whois.nic.md",
    "me",	"whois.nic.me",
    "mg",	"whois.nic.mg",
    "mh",	"\x03",
    "mk",	"whois.marnet.mk",
    "ml",	"whois.dot.ml",
    "mm",	"\x03",
    "mn",	"whois.nic.mn",
    "mo",	"\x01http://www.monic.mo/",
    "mp",	"\x03",
    "mq",	"\x01https://www.dom-enic.com/whois.html",
    "mr",	"\x03",
    "ms",	"whois.nic.ms",
    "mt",	"\x01https://www.nic.org.mt/dotmt/",
    "mu",	"whois.nic.mu",
    "mv",	"\x03",
    "mw",	"\x01http://www.registrar.mw/",
    "mx",	"whois.mx",
    "my",	"whois.mynic.my",
    "mz",	"whois.nic.mz",
    "na",	"whois.na-nic.com.na",
    "nc",	"whois.nc",
    "ne",	"\x03",
    "nf",	"whois.nic.nf",
    "ng",	"whois.nic.net.ng",
    "ni",	"\x01http://www.nic.ni/",
    "nl",	"whois.domain-registry.nl",
    "no",	"whois.norid.no",
    "np",	"\x01http://register.mos.com.np/np-whois-lookup",
    "nr",	"\x01http://www.cenpac.net.nr/dns/whois.html",
    "nu",	"whois.iis.nu",
    "nz",	"whois.srs.net.nz",
    "om",	"whois.registry.om",
    "pa",	"\x01http://www.nic.pa/",
    "pe",	"kero.yachay.pe",
    "pf",	"whois.registry.pf",
    "pg",	"\x03",
    "edu.ph",	"\x01http://services.ph.net/dns/query.pl",
    "gov.ph",	"\x03",
    "ph",	"\x01http://www.dot.ph/whois",
    "pk",	"\x01http://www.pknic.net.pk/",
    "co.pl",	"whois.co.pl",
    "pl",	"whois.dns.pl",
    "pm",	"whois.nic.pm",
    "pn",	"\x01http://www.pitcairn.pn/PnRegistry/",
    "pr",	"whois.nic.pr",
    "ps",	"whois.pnina.ps",
    "pt",	"whois.dns.pt",
    "pw",	"whois.nic.pw",
    "py",	"\x01http://www.nic.py/consulta-datos.php",
    "qa",	"whois.registry.qa",
    "re",	"whois.nic.re",
    "ro",	"whois.rotld.ro",
    "rs",	"whois.rnids.rs",
    "edu.ru",	"whois.informika.ru",
    "ru",	"whois.tcinet.ru",
    "rw",	"whois.ricta.org.rw",
    "sa",	"whois.nic.net.sa",
    "sb",	"whois.nic.sb",
    "sc",	"whois2.afilias-grs.net",
    "sd",	"\x03",
    "se",	"whois.iis.se",
    "sg",	"whois.sgnic.sg",
    "sh",	"whois.nic.sh",
    "si",	"whois.register.si",
    "sj",	"\x03",
    "sk",	"whois.sk-nic.sk",
    "sl",	"whois.nic.sl",
    "sm",	"whois.nic.sm",
    "sn",	"whois.nic.sn",
    "so",	"whois.nic.so",
    "sr",	"\x03",
    "st",	"whois.nic.st",
    "su",	"whois.tcinet.ru",
    "sv",	"\x01http://www.svnet.org.sv/",
    "sx",	"whois.sx",
    "sy",	"whois.tld.sy",
    "sz",	"\x03",
    "tc",	"whois.nic.tc",
    "td",	"\x01http://www.nic.td/",
    "tf",	"whois.nic.tf",
    "tg",	"whois.nic.tg",
    "th",	"whois.thnic.co.th",
    "tj",	"\x01http://www.nic.tj/whois.html",
    "tk",	"whois.dot.tk",
    "tl",	"whois.nic.tl",
    "tm",	"whois.nic.tm",
    "tn",	"whois.ati.tn",
    "to",	"whois.tonic.to",
    "tr",	"whois.nic.tr",
    "tt",	"\x01http://www.nic.tt/cgi-bin/search.pl",
    "tv",	"\x04" "tvwhois.verisign-grs.com",
    "tw",	"whois.twnic.net.tw",
    "tz",	"whois.tznic.or.tz",
    "biz.ua",	"whois.biz.ua",
    "co.ua",	"whois.co.ua",
    "pp.ua",	"whois.pp.ua",
    "ua",	"whois.ua",
    "ug",	"whois.co.ug",
    "ac.uk",	"whois.ja.net",
    "bl.uk",	"\x03",
    "british-library.uk",	"\x03",
    "gov.uk",	"whois.ja.net",
    "icnet.uk",	"\x03",
    "jet.uk",	"\x03",
    "mod.uk",	"\x03",
    "nhs.uk",	"\x03",
    "nls.uk",	"\x03",
    "parliament.uk",	"\x03",
    "police.uk",	"\x03",
    "uk",	"whois.nic.uk",
    "fed.us",	"whois.nic.gov",
    "us",	"whois.nic.us",
    "com.uy",	"\x01https://nic.anteldata.com.uy/dns/consultaWhois/whois.action",
    "uy",	"whois.nic.org.uy",
    "uz",	"whois.cctld.uz",
    "va",	"\x03",
    "vc",	"whois2.afilias-grs.net",
    "ve",	"whois.nic.ve",
    "vg",	"whois.nic.vg",
    "vi",	"\x01https://secure.nic.vi/whois-lookup/",
    "vn",	"\x01http://www.vnnic.vn/en/domain",
    "vu",	"vunic.vu",
    "wf",	"whois.nic.wf",
    "ws",	"whois.website.ws",
    "ye",	"\x03",
    "yt",	"whois.nic.yt",
    "ac.za",	"whois.ac.za",
    "alt.za",	"whois.alt.za",
    "co.za",	"whois.registry.net.za",
    "gov.za",	"whois.gov.za",
    "net.za",	"net-whois.registry.net.za",
    "org.za",	"org-whois.registry.net.za",
    "web.za",	"web-whois.registry.net.za",
    "za",	"\x03",
    "zm",	"whois.nic.zm",
    "zw",	"\x03",
    "xn--3e0b707e",	"whois.kr",
    "xn--45brj9c",	"whois.inregistry.net",
    "xn--54b7fta0cc",	"\x03",
    "xn--80ao21a",	"whois.nic.kz",
    "xn--90a3ac",	"whois.rnids.rs",
    "xn--90ae",	"whois.imena.bg",
    "xn--90ais",	"whois.cctld.by",
    "xn--clchc0ea0b2g2a9gcd",	"whois.sgnic.sg",
    "xn--d1alf",	"whois.marnet.mk",
    "xn--e1a4c",	"whois.eu",
    "xn--fiqs8s",	"cwhois.cnnic.cn",
    "xn--fiqz9s",	"cwhois.cnnic.cn",
    "xn--fpcrj9c3d",	"whois.inregistry.net",
    "xn--fzc2c9e2c",	"whois.nic.lk",
    "xn--gecrj9c",	"whois.inregistry.net",
    "xn--h2brj9c",	"whois.inregistry.net",
    "xn--j1amh",	"whois.dotukr.com",
    "xn--j6w193g",	"whois.hkirc.hk",
    "xn--kprw13d",	"whois.twnic.net.tw",
    "xn--kpry57d",	"whois.twnic.net.tw",
    "xn--l1acc",	"\x03",
    "xn--lgbbat1ad8j",	"whois.nic.dz",
    "xn--mix891f",	"\x01http://www.monic.mo/",
    "xn--mgb9awbf",	"whois.registry.om",
    "xn--mgba3a4f16a",	"whois.nic.ir",
    "xn--mgbaam7a8h",	"whois.aeda.net.ae",
    "xn--mgbayh7gpa",	"\x01http://idn.jo/whois_a.aspx",
    "xn--mgbbh1a71e",	"whois.inregistry.net",
    "xn--mgbc0a9azcg",	"\x03",
    "xn--mgberp4a5d4ar",	"whois.nic.net.sa",
    "xn--mgbpl2fh",	"\x03",
    "xn--mgbtx2b",	"whois.cmc.iq",
    "xn--mgbx4cd0ab",	"whois.mynic.my",
    "xn--node",	"whois.itdc.ge",
    "xn--o3cw4h",	"whois.thnic.co.th",
    "xn--ogbpf8fl",	"whois.tld.sy",
    "xn--p1ai",	"whois.tcinet.ru",
    "xn--pgbs0dh",	"\x03",
    "xn--qxam",	"\x01https://grweb.ics.forth.gr/public/whois.jsp?lang=en",
    "xn--s9brj9c",	"whois.inregistry.net",
    "xn--wgbh1c",	"whois.dotmasr.eg",
    "xn--wgbl6a",	"whois.registry.qa",
    "xn--xkc2al3hye2a",	"whois.nic.lk",
    "xn--xkc2dl3a5ee0h",	"whois.inregistry.net",
    "xn--y9a3aq",	"whois.amnic.net",
    "xn--yfro4i67o",	"whois.sgnic.sg",
    "xn--ygbi2ammx",	"whois.pnina.ps",
